#pragma once
#include "ap_pal.h"

#include "DMClient.h"
#include "DMClientSettings.h"

namespace dmclient
{
    /// <summary>
    /// The methods in this class are used only from test code. The methods here provide access
    /// to private fields in the DMClient classes.
    /// </summary>
    class DMClientTestHelper
    {
    public:

        /// <summary>
        /// Inits the DMClient with the provided DMClientSettings objects (so that the DMClient code doesn't attempt to read
        /// these from shared.ini / environment.ini / etc - which we won't have in tests).
        /// </summary>
        static bool Init(_In_ apsdk::DMClient& dmClient, _In_ std::shared_ptr<DMClientSettings> clusterSettings, _In_ std::shared_ptr<DMClientSettings> environmentSettings, _In_ std::shared_ptr<DMClientSettings> applicationSettings);

        /// <summary>
        /// Inits the DMClient with the provided DMClientSettings objects (so that the DMClient code doesn't attempt to read
        /// these from shared.ini / environment.ini / etc - which we won't have in tests).
        /// </summary>
        static bool Init(
            _In_ apsdk::DMClient& dmClient,
            _In_ std::shared_ptr<DMClientSettings> clusterSettings,
            _In_ std::shared_ptr<DMClientSettings> environmentSettings,
            _In_ std::shared_ptr<DMClientSettings> applicationSettings,
            _In_ apsdk::DMClient::Handler* handler);

        /// <summary>
        /// Inits the DMClient with the provided DMClientSettings objects (so that the DMClient code doesn't attempt to read
        /// these from shared.ini / environment.ini / etc - which we won't have in tests).
        /// </summary>
        static bool Init(
            _In_ apsdk::DMClient& dmClient,
            unsigned int sendTimeoutMilliseconds,
            unsigned int receiveTimeoutMilliseconds,
            _In_ std::shared_ptr<DMClientSettings> clusterSettings,
            _In_ std::shared_ptr<DMClientSettings> environmentSettings,
            _In_ std::shared_ptr<DMClientSettings> applicationSettings,
            _In_ apsdk::DMClient::Handler* handler);

        /// <summary>
        /// Invoking this causes DMClient to re-create the internal DMClientImpl object, based on the settings values.
        /// This is needed for test code because in test code, we are not using the underlying shared.ini / environment.ini files,
        /// and so there is no change notification listener which would trigger DMClient to re-read the settings.
        /// </summary>
        static void NotifySettingsChanged(_In_ apsdk::DMClient& dmClient);

        /// <summary>
        /// Updates the settings object, setting the private fields.
        /// </summary>
        static void UpdateDmGatewaySettings(_In_ DMClientSettings* settings, _In_opt_z_ const char* dmGatewayResourceUrl, _In_opt_ int* dmGatewayPort, _In_opt_ bool* useHttps);

        /// <summary>
        /// Updates the environment settings object, setting the private fields.
        /// </summary>
        static void UpdateRetrySettings(_In_ DMClientSettings* settings, _In_opt_ int* numRetries);

        /// <summary>
        /// Creates a DMClientSettings object with default values. We have this here, since
        /// in the product code, the only public methods are the ones that create this object
        /// using either shared.ini / environment.ini / application default config.
        /// </summary>
        static DMClientSettings* GetDefaultSettings();

        /// <summary>
        /// Helper method to get the list of dmgateway nodes populated after initialization with current input settings.
        /// </summary>
        /// <param name="dmGatewayMachines">Output DMGateway machines to connect to for sending requests.</param>
        /// <param name="dmGatewayVip">DMGateway vip</param>
        /// <param name = "applicationSettings">application level settings</param>
        /// <param name = "environmentSettings">environment level settings</param>
        /// <param name = "clusterSettings">cluster level settings</param>
        /// <returns>True on success, false on failure.</returns>
        static bool GetDMGatewayNodesHelper(
            _Out_ std::vector<std::string>& dmGatewayMachines,
            _Out_ std::string& dmGatewayVip,
            _In_opt_ std::shared_ptr<dmclient::DMClientSettings> applicationSettings,
            _In_opt_ std::shared_ptr<dmclient::DMClientSettings> environmentSettings,
            _In_opt_ std::shared_ptr<dmclient::DMClientSettings> clusterSettings);
    };
}
